<?

/*
 * ConfigModule:
 * This module is used to dynamically insert modules that are needed for a
 * specific form. There are 2 ways of including these modules:
 * 1. AUTO_CONFIGURATION: all modules are scanned (*.def.xml) for options
 * that belong to a specific module. If the option is used in the form the
 * module is loaded. The routine for this auto-configuration is pretty
 * slow, so it is possible to turn it off in configfile.mod.php
 * 2. options 'modules' Every module that is added to the modules-option, is
 * loaded (multiple options can be seperated with :)
 */

define("MOD_CONFIG",$name);
define("CONFIG_SEP",":");

// Options
define("EXTRA_MODULES",0);
define("CONFIGFILE",1);

$modules[$name]= new ConfigModule();

/**
 * class ConfigModule
 * this class takes some options to add extra modules or to add config
 * options via a configfile
 *
 * @author    Herman Suijs (<a href="mailto:php@suijsmail.com">php@suijsmail.com</a>)
 * @version   1.6a
 */

class ConfigModule extends Module {
	var $name = "ConfigModule";
	var $id = MOD_CONFIG;
	var $execlevel = 1;
	var $modules = "";

	function ConfigModule() {
		$this->options[EXTRA_MODULES] = new Option("config:modules:extra_modules","",false,false);
		$this->options[CONFIGFILE] = new Option("configfile:config_file");
		$this->Module();
	}
	
	function before_action() {
global $formHandler,$debug;
		$this->check_options();
		// If a configfile is given, load it in
		if ($this->options[CONFIGFILE]) {
			$configfile = file(CONFIG_DIR."/".$this->options[CONFIGFILE]->value);
			unset($this->options[CONFIGFILE]);
			while (list(,$line)=each($configfile)) {
				$line_info = ereg_replace("(#|\/\/|\/\*).*$","",$line);
				if (!strpos($line_info,"=")===false) {
					list($config_name,$config_value)=split("=",$line_info);
					// Make sure the debug-options also work
					if ($config_name=="config"||$config_name=="modules"||$config_name=="extra_modules") {
						$this->modules = $config_value;
					} elseif ($config_name=="debug") {
						$debug->enable(is_true($config_value));
						//$debug->add_message("Option: " . $config_name . " ". $config_value,$this->name);
					} elseif ($config_name=="noop") {
						$debug->set_noop(is_true($config_value));
						//$debug->add_message("Option: " . $config_name . " ". $config_value,$this->name);
					} elseif (!strpos($config_name,"level")===false) {
						$debug->set_level($config_value);
					}
						$debug->add(5,$this->name,"Option: %s %s",$config_name,$config_value);
					$config[$config_name] = $config_value;
				} else {
					$debug->add(7,$this->name,"Comment or ... in configfile: %s",  $line);
				}
			}
			unset($configfile);
			// add the values of the configfile to the vars-array
			//if (!empty($config)) $formHandler->set_default_vars($config);
				$formHandler->addto_vars($config,CONFIGFILE_OVERRIDE);
		}
	}

	// Because the file xml.inc.php is quite large and may downgrade the
	// performance of the whole script, you can turn of auto-configuration in
	// the configfile.mod.php (must become xml config one day)
	function auto_config() {
	// return an array with modules to add
		global $debug,$formHandler;
		// Use all vars for configuration
		$vars = $formHandler->get_vars();
		//$vars = $formHandler->get_showvars();
		//echo fh_serialize($vars);
		$load_modules = array();
		// Include class-file for reading xml-config files
		include_once("xml.inc.php");

		$localdir = opendir(LOCAL_DIR);
		while ($xmlfile = readdir($localdir)) {
			if (ereg(DEF_EXT ."$",$xmlfile)) {
				$xml = new XML($xmlfile);
				$load_in = false;
				//$debug->add_message("XML-file: $xmlfile",$this->name."(AUTO_CONFIG)");
				$debug->add(3,$this->name."(AUTO_CONFIG)","XML-file: %s",$xmlfile);
				$module_path = "/module[1]";
				$mod_name = $xml->get_content($module_path . "/name[1]");
				$mod_file = $xml->get_content($module_path . "/file[1]");
				$options = $xml->evaluate($module_path . "/options/optionname");
				//$debug->add_mesage(fh_serialize($options) ."<br>",$this->name);
				foreach ($options as $option) {
					if ($load_in) break;
					$optionname = $xml->get_content($option);
					$attributes = $xml->get_attributes($option);
					if ($attributes["aliases"] != "")
						$aliases = split(":",$attributes["aliases"]);
					$aliases[] = $optionname;
					//echo fh_serialize($aliases) ."<br>";
					while (list(,$alias) = each($aliases)) {
						if ($vars[$alias] !="") {
							$load_modules[$mod_name] = $mod_file;
							//$debug->add_message("Module $mod_name ($optionname-$alias): $mod_file <br>",$this->name);
							$debug->add(6,$this->name,"Module %s (%s): %s", $mod_name,"$optionname-$alias",$mod_file,$this->name);
							$load_in=true;
							break;
						}
					}
				}
				unset($xml);
			}
		}
		return $load_modules;
	}

	function do_action() {
		global $formHandler,$modules,$debug,$name;
		$this->action();

		reset($this->options);
		//echo fh_serialize($this->options);
		//list($name,$config) = each($this->options);
		$config = "";
		$sep = false;
		if ($this->options[EXTRA_MODULES]) {
			if ($sep) $config.= CONFIG_SEP;
			$config .= $this->options[EXTRA_MODULES]->value;
			$sep = true;
		}
		if (!empty($this->modules)) {
			if ($sep) $config.= CONFIG_SEP;
			$config .= $this->modules;
			$sep = true;
		}
		//echo "addModule $name " .fh_serialize($config) . "<br>";
		if (!strpos($config,CONFIG_SEP)===false) {
			$configs = split(CONFIG_SEP,$config);
			while (list(,$name) = each($configs)) {
				$name = trim($name);
				if (!$added[$name]) {
					$message = addModule($name,$name);
				}
				$added[$name] = true;
				//$debug->add_message($message,$this->name);
				$debug->add(6,$this->name,$message);
			}
		} else {
			$message = addModule($config,$config);
			//$debug->add_message($message,$this->name);
			$debug->add(6,$this->name,$message);
		}

		if ($formHandler->config(CFG_AUTO_CONFIG)) {
			//$debug->add_message("AUTO_CONFIG",$this->name);
			$debug->add_message(6,$this->name,"AUTO_CONFIG");
			$load_modules = $this->auto_config();
			while (list($name,$file) = each($load_modules)) {
				$message = addModule($file,$name);
				//$debug->add_message($message,$this->name);
				$debug->add(6,$this->name,$message);
			}
		}

	}

	function after_action() {
		$this->clean_up();
	}
}

