<?
/* 
 * FormHandler basic script.
 * This is the script that is actually called and does the initiation of
 * the basic modules and classes.
 */

// The action-levels are necessary, because otherwise the deletion of
// options configured in the form will be done AFTER the mail is sent.
/* This is the main file of the form-handler package.
 */

// Check for a different config-file option from the form and load that
// configuration file. Otherwise use the default configuration.
// This is the only option in a form that is not handled by a module.
$starttime=time();
if ($config_file) {
	require_once "$config_file";
}
else {
	require_once "configfile.inc.php";
}

// Global array of modules that are loaded and can be used.
$modules = array(); // Always globally addressed
$wrap_up_mods = array(); // Modules that have a wrap_up function.
$debug = false; // for checking existence
$error = false;
$tmp_debug_array = array();
$runlevel = 0;
$formHandler = false;


// This function Sorts all the dynamically loaded modules, do they will be
// executed in the runlevel they specified.
function createSortedArray($modules) {
	global $debug;
	$sorted_modules = array();
	reset($modules);
	while (list($name,$module) = each($modules)) {
		$execlevel = $module->execlevel();
		if (isset($execlevel)) {
			if ($sorted_modules[$execlevel]) {
				$sorted_modules[$execlevel] .= ":" . $name;
			} else {
				$sorted_modules[$execlevel] = $name;
			}
		} // end if isset
	}
	ksort($sorted_modules);
	reset($sorted_modules);
	while (list($name,$val) = each($sorted_modules)) {
		//$debug->add_message("$name - $val","createSortedArray");
		$debug->add(6,"FH_createSortedArray","%s - %s",$name,$val);
	}
	reset($sorted_modules);
	return $sorted_modules;
}

// Dynamically load in a module
function addModule($module_file,$name,$init=false) {
	global $runlevel,$modules;
	// Module should create itself and a constant with name
	$module_file = ereg("php$",$module_file) ? $module_file : $module_file . MOD_EXT;
	require_once "$module_file";
	// Debug-message before debug-object exists
	$message="Module $name ($module_file) added.";
	//echo "<br>";
	
	// Adjust runlevel if it is too small for a module that has been created
	if (isset($modules[$name])) {
		$exec = $modules[$name]->execlevel();
		if (!empty($exec)&&$exec<$runlevel*STEP_EXECLEVEL) {
			// Execute at first in the next runlevel run
			$modules[$name]->set_execlevel(($runlevel+1)*STEP_EXECLEVEL);
		} // end if empty
	} // end if isset
	return $message;
}

// Check for false options
function is_true($option) {
	if (eregi(_FALSE,$option)) {
		return false;
	} else {
		return true;
	}
}

// Rewrite serialize function
function fh_serialize($array) {
	if (is_array($array)) {
		$return_string = "<ul>\n";
		while (list($name,$value) = each($array)) {
			if (is_array($value)) {
				$return_string .= "<li>$name: "+fh_serialize($value)+"\n";
			} else {
				$return_string .= "<li>$name: $value\n";
			}
		}
		$return_string .= "</ul>\n";
	} else {
		$return_string = $array;
	}
	return $return_string;
}

/****************************************************************/


// Load in the (pre) configured modules, these are defined in
// configfile.inc.php
reset($load_modules);
while(list($name,$module_file) = each($load_modules)) {
	$message = addModule($module_file,$name,true);
	// Debugmessage
	if ($debug) {
		//$debug->add_message($message,"Init");
		$debug->add(-1,"FH_Init",$message);
	} else {
		// Store in temp-array if debug object doesn't exist
		$tmp_debug_array[$name] = $message;
	}

  // Make reporting objects globally addressable
	switch ($name) {
		case MES_DEBUG :
			$debug = &$modules[MES_DEBUG];
			// Add the debug-messages so far.
			$debug->message_array($tmp_debug_array);
			unset($tmp_debug_array); // Destroy the variable
			break;
		case MES_ERROR :
			$error = &$modules[MES_ERROR];
			break;
	}

}

// Global addressable exit function
// This should call functions from different modules and debug and error
function exit_fh() {
	global $modules,$wrap_up_mods,$debug,$starttime;
	//$debug->add_message("Exit function called","exit_fh function",-1);
	$debug->add(-1,"FH_exit_fh function","Exit function called");
	$endtime=time();
	$time=$endtime-$starttime;
	$debug->add(-1,"FH_exit_fh function","Execution time: %s sec",$time);
	reset($wrap_up_mods);
	while (list($module_name,$func) = each($wrap_up_mods)) {
		//echo "Exit function ($func) for module $module_name ($modules[$module_name])called<br>";
		$modules[$module_name]->$func();
	}
	// This is automatically done by the above routine.
	//$GLOBALS['debug']->wrap_up();
	//$GLOBALS['error']->wrap_up();
	//$time=time()-$starttime;
	//echo "Execution time: $time sec";
	exit;
}

// The actual functionality of the script is dynamically loaded in as a
// module. The formHandler variable gives all other modules a reference to
// that module.
$formHandler =& $modules[FORMHANDLER];
//echo serialize($formHandler);
$formHandler->set_html(true,ERROR_CONTENT);
$formHandler->set_title("Error",ERROR_CONTENT);
$formHandler->set_html(true,DEBUG_CONTENT);
$formHandler->set_header("<h2>Debug</h2>",DEBUG_CONTENT);
$formHandler->set_footer("",DEBUG_CONTENT);

// Start runlevels
for($runlevel=0;$runlevel<MAX_STEPS;$runlevel++) {
	$startrltime=time();
	$formHandler->runlevel($runlevel);
	$rltime=time()-$startrltime;
	$debug->add(-1,"FH Runlevel steps","Execution time runlevel %s: %s sec",$runlevel,$rltime);
}
// call the exit function to properly wrap-up everything.
exit_fh();

