<?

/*
 * FormHandler:
 * This module creates the different modules that need to be used. It
 * contains the text to be used in the mail, display, etc. It distributes
 * the variables entered in the form and it creates the runlevels and runs
 * the different modules.
 */

define("FORMHANDLER",$name);

define("EXEC_MODULE_SEP",":");

include_once("vars.inc.php");
include_once("content.inc.php");
include_once("configopts.inc.php");

global $REQUEST_METHOD,$HTTP_GET_VARS,$HTTP_POST_VARS;
// Storing the name of the array instead of the array saves memory
$http_vars = ($REQUEST_METHOD=="GET") ? HTTP_GET_VARS : HTTP_POST_VARS;

// Create the FormHandler object and make it run.
$modules[FORMHANDLER] = new FormHandler($$http_vars, $HTTP_COOKIE_VARS);
unset($http_vars);

class FormHandler extends ModuleBase {
	var $name = "FormHandler";
	var $id = FORMHANDLER;
	var $opties = array();
	var $vars = array(); // complete array of HTTP_VARS
	//var $show_vars = array(); // array of vars to show in mail(s)
	//var $cookies = array();
	var $content = array(); // strings to use for mailing and displaying
	//var $mail_content = ""; // string to use for mailing
	//var $display_content = ""; // string to use for displaying
	var $sorted_modules = array();
	var $config;
	var $translator;

	// Initialize the Handlerobject
	function FormHandler($vars,$cookies) {
		global $modules,$error;
		$this->translator = new Translator(TRANSLATE_BY_DEFAULT);
		$this->ModuleBase();
		$this->config = new Config();
		//$this->content[CONTENT] = new Content(CONTENT);

		if (!is_array($vars)||empty($vars)) {
			$error->add_message("No variables submitted. Did you use a form ?",$this->name,3);
			break;
		}
		//$this->vars = $vars;
		$this->vars = new Vars_Array($vars);

		/*
		while (list($name,$value) = each($vars)) {
			$this->show_vars[strtolower($name)] = $value;
		}
		*/
		
		//$this->cookies = $cookies;
		if (is_array($cookies)&&!empty($cookies)) {
			$this->vars->addto_vars($cookies);
			$this->vars->set_vartype(array_keys($cookies),USR_VARS);
			$this->vars->set_vartype(array_keys($cookies),COOKIE_VARS);
		}

		$this->sorted_modules = createSortedArray($modules);

	}

	// For compatibility with other modules
	function execlevel() {
		return 0; // First execlevel. Is ignored
	}

	function translate($message,$var1="",$var2="",$var3="",$var4="",$var5="",$var6="",$var7="",$var8="",$var9="",$var10="",$var11="",$var12="") {
		return $this->translator->translate($message,$var1="",$var2="",$var3="",$var4="",$var5="",$var6="",$var7="",$var8="",$var9="",$var10="",$var11="",$var12="");
	}

	// Method to create an error
	/*
	function error($name,$level,$var1,$var2="",$var3="",$var4="",$var5="") {
		if (!is_array($var1)) {
			$vars = array($var1,$var2,$var3,$var4,$var5);
		} else {
			$vars=$var1;
		}

	}
	*/


	// *************  Execution of modules
	// The action-levels are necessary, because otherwise the deletion of
	// options configured in the form will be done AFTER the mail is sent.
	function exec_module($module, $action) {
		global $modules,$debug;
		//$debug->add_message("Running module action $action: $module",$this->name);
		$debug->add(DB_XFLOW,$this->name,"Running module action %s: %s",$action,$module);
		$module =& $modules[$module];
		$skip = $module->enabled;
		if (!$skip) {
			$debug->add(DB_XFLOW,$this->name,"Module %s is not enabled",$module);
		}
		switch ($action) {
			case 0 :
				$module->before_action();
				break;
			case 1 :
				if ($skip) $module->do_action();
				break;
			case 2 :
				$module->after_action();
				break;
		}
		unset($module);
		return $skip; // Return if module-action is executed
	}

	function runlevel($levelgroup) {
		global $modules,$debug;
		//$debug->add_message("Entering runlevel $levelgroup",$this->name);
		$debug->add(3,$this->name,"Entering runlevel %s",$levelgroup);
		for($action=0; $action<MAX_ACTION; $action++) {
			for($level=0;$level<STEP_EXECLEVEL; $level++) {
				$exec_module = $this->sorted_modules[$level+$levelgroup*STEP_EXECLEVEL];
				//echo "$levelgroup $level $exec_module <br>";
				if (!empty($exec_module)) {
					if (!strpos($exec_module,EXEC_MODULE_SEP)===false) {
						$exec_module = split(EXEC_MODULE_SEP,$exec_module);
						while (list(,$module) = each($exec_module)) {
							$this->exec_module($module,$action);
						}
					} else {
						$this->exec_module($exec_module,$action);
					} // end if (strpos)
				} // end if (empty)
			} // end for level
		} // end for step
		//$debug->add_message("Leaving runlevel $levelgroup",$this->name);
		$debug->add(6,$this->name,"Leaving runlevel %s",$levelgroup);
		$this->sorted_modules = createSortedArray($modules);
	} // end function


// *********** Config code
	// Method to retrieve config-information
	function config($option) {
		return $this->config->getOption($option);
	}

// *********** Content code
	/*
	function addto_content($name,$content) {
		$this->content[$name]->addto_content($content);
	}
	*/
	// Functions for content communications
	function set_html($html=true,$type=CONTENT) {
		if (empty($this->content[$type])) {
			$this->content[$type] = new Content($type);
		}
		$this->content[$type]->set_html($html);
	}

	function convert_content($vars,$type=CONTENT) {
		if (empty($this->content[$type])) {
			$this->content[$type] = new Content($type);
		}
		$this->content[$type]->convert_content($vars);
	}

	function set_content($new_content, $type=CONTENT) {
		global $error,$debug;
		if (!isset($this->content[$type])) {
			$this->content[$type] = new Content($type,$new_content);
		} else {
			$this->content[$type]->set_content($new_content);
		}

		$debug->add(8,$this->name,"New content (%s) set",$type);
		return true;

	}

	function addto_content($add_content, $type=CONTENT) {
		if (empty($this->content[$type])) {
			$this->set_content($type,$add_content);
		} else {
			$this->content[$type]->addto_content($add_content);
		}
	}

	function insert_content($ins_content, $type=CONTENT) {
		if (empty($this->content[$type])) {
			$this->set_content($type,$ins_content);
		} else {
			$this->content[$type]->insert_content($ins_content);
		}
	}

	/*
	function insert_content($ins_content) {
		$this->text = $ins_content . $this->content;
	}
	*/

	function get_content($type=CONTENT,$final=false,$vars="") {
		global $debug;
		if (!empty($this->content[$type])) {
			if ($final) {
				$debug->add(8,$this->name." get-content","Get final %s","Content[$type]");
			}
			return $this->content[$type]->get_content($final,$vars);
		} else {
			$debug->add(8,$this->name." get-content","%s not defined yet","Content[$type]");
			return "";
		}
	}

	function set_header($header,$type=CONTENT) {
		global $debug;
		if (!empty($this->content[$type])) {
			$this->content[$type]->set_header($header);
		} else {
			$debug->add(8,$this->name." set_header","%s not defined yet","Content[$type]");
		}

	}

	function set_footer($footer,$type=CONTENT) {
		global $debug;
		if (!empty($this->content[$type])) {
			$this->content[$type]->set_footer($footer);
		} else {
			$debug->add(8,$this->name." set_footer","%s not defined yet","Content[$type]");
		}

	}

	function set_title($title,$type=CONTENT) {
		global $debug;
		if (!empty($this->content[$type])) {
			if (empty($title)) {
				$this->content[$type]->set_title_default($title);
			} else {
				$this->content[$type]->set_title($title);
			}
		} else {
			$debug->add(8,$this->name." set_title","%s not defined yet","Content[$type]");
		}
	}

	/*
	function content() {
		// This function is going to be phased out.
		return $this->get_content();
	}
	*/

	function isset_content($type=CONTENT) {
		// return a boolean
		return $this->content[$type]->isset_content();
	}


// *********** Vars code
	// Functions for variable communications
	function get_vars($type=VARS) {
		return $this->vars->get_vars($type);
	}

	function set_vars($vars) {
		$this->vars->set_vars($vars);
	}

	function addto_vars($vars,$replace=false) {
		global $error;
		if (is_array($vars)) {
			//$this->vars[$name]->addto_vars($vars,$replace);
			$this->vars->addto_vars($vars,$replace);
		} else {
			$error->add(1,$this->name."addto_vars","No array used");
		}
	}

	function change_vars($name,$var_name,$var_value="") {
		$this->addto_vars($name,$var_name,$var_value);
	}

	function del_vars($name,$var_name) {
		$this->vars[$name]->del_vars($var_name);
	}

	/*
	function get_vars() {
		return $this->vars;
	}

	function set_default_vars($config) {
		global $debug;
		while (list($name,$value) = each($config)) {
			if (!isset($this->vars[$name])) {
				$this->vars[$name] = $value;
				//$debug->add_message("Value of vars[$name] set to filevalue: $value",$this->name);
				$debug->add(6,$this->name,"Value of %s set to filevalue: %s","vars[$name]",$value);
			}
		}
	}

	function addto_vars($name,$value) {
		global $debug;
		if (isset($this->vars[$name])) {
			//$debug->add_message("Value of vars[$name] changed to $value",$this->name);
			$debug->add(6,$this->name,"Value of %s changed to $value","vars[$name]",$value);
		}
		$this->vars[$name]=$value;
	}

	function get_showvars() {
		return $this->show_vars;
	}
	*/

	function del_var($key) {
		global $debug;
		if (!$debug->enabled) {
			unset($this->show_vars[$key]);
		}
		//$debug->add_message("Delete var $key",$this->name);
		$debug->add(6,$this->name,"Delete var %s",$key);
	}
	
	// Functions for cookies communications
	/*
	function cookies() {
		return $this->cookies;
	}
	*/

}

