<?
/*
 * Module:
 * This class is an interface which contains the minimum
 * function-set that is needed for a module to work with the form-handler
 * package.
 *
 * Option:
 * This class represents a field that is used in a form. The option can be
 * created in a module and at the execution of the module it checks the
 * existence and the value of the option.
 */

define("MOD_MODULE",$name);
define("ALIAS_SEP",":");

// This is probably not usefull, because Module is an interface class
//$modules[$name]= new Module; 

class Module extends ModuleBase {
	var $name = "Module";
	var $options = array();
	var $execlevel;
	//var $error_messages = array();

	function Module() {
		global $error,$debug;
		if (empty($this->execlevel)) {
			//$error->add_message("No execlevel",$this->name,3);
			$error->add(ERR_DEV_END,$this->name,"No execlevel");

		}

		if (empty($this->options)) {
			//$debug->add_message("No options defined (yet).",$this->name,3);
			$debug->add(DB_WARN,$this->name,"No options defined (yet).");
		} //else {
			//$this->check_options();
		//}
		// Add options

		$this->ModuleBase();
	}

	function execlevel() {
		return $this->execlevel;
	}

	function set_execlevel($level) {
		global $debug;
		$max_level = STEP_EXECLEVEL*MAX_STEPS;
		if ($level>=$max_level) {
			$level = $max_level;
			//$debug->add_message("Execlevel $level>$max_level. Changed to $max_level.");
			$debug->add(DB_FLOW,$this->name,"Execlevel %s>%s. Changed to %s.",$level,$max_level,$max_level);
		}
		$this->execlevel = $level;
	}

	function check_options() {
		global $debug,$formHandler;
		//$vars = $formHandler->get_showvars();
		$vars = $formHandler->get_vars();
		//echo "$this->name: ".fh_serialize($vars);
		$thisname = $this->name;
		reset($this->options);
		// This value needs to be set seperately, because the value of
		// count($this->options) changes by the unset statement.
		$max = count($this->options);
		// This is needed, because $option needs to be a reference to the
		// array-element of $this->options. Otherwise the changes to the
		// specific options-object made in one of the methods (functions of the
		// option) is made in a copy and discarded after this routine.
		for ($id=0; $id < $max; $id++) {
			if (!empty($this->options[$id])) {
				$option =& $this->options[$id];
				//echo("Check option ".$option->name);
				//$debug->add_message("Check option ".$option->name,$thisname);
				$debug->add(DB_XFLOW,$thisname,"Check option %s",$option->name);
				if($option->check_existence($vars)) {
					//$debug->add_message("Check value option ".$option->name,$thisname);
					$debug->add(DB_XFLOW,$thisname,"Check value option %s",$option->name);
					$option->check_value($vars);
					//$debug->add_message("Option($id): " . $option->name . " ". $option->value,$thisname);
					$debug->add(DB_XFLOW,$thisname,"Option(%s): %s %s",$id,$option->name,$option->value);
				} else {
					//echo "$thisname $optionname ". $option->enabled ."<br>";
					if (!$option->enabled) {
						$this->disable();
						//$debug->add_message("Module ".$this->name." is disabled",$thisname);
						$debug->add(DB_XFLOW,$thisname,"Module %s is disabled",$this->name);
					}
					$debug->add(DB_XFLOW,$thisname,"Option %s is unset",$option->name);
					// Just to be sure, first set to NULL, then unset
					$this->options[$id] = NULL;
					unset($this->options[$id]);
					//echo count($this->options) ." ".$id ."<br>";
				}
				//echo "Option($id): " . $option->name . " ". $option->value ."<br>";
	
				// Unset must be at the end, because otherwise the total option is
				// discarded when cleaning up this local variable.
				// Probably a bug in PHP at this time.
				unset($option);
			} //end if isset
			else $max++;
		} // end for
	}

	function action() {
		//global $debug,$error;
		//return (!$debug->noop()&&!$error->noop());
		global $debug,$error;
		$message= $this->name . " no action defined (yet) or super called: ";
		$message.= "debug " . fh_serialize($debug->noop());
		$message.= "error " . fh_serialize($error->noop());
		//$message.= "<BR>";
		$debug->add(DB_INFO,$this->name,$message);
		return (!($debug->noop()||$error->noop()));
	}

	function clean_up() {
		global $modules;
		reset($this->options);
		while (list($name,) = each($this->options)) {
			unset($options[$name]);
		}
		if (!$this->enabled) {
			unset($modules[$this->id]);
		}
	}

	//function return_error_message($number) {
	//}

} // class Module


class Option extends ModuleBase {
	//var $name; // Defined in ModuleBase
	//var $id; // Defined in ModuleBase
	var $aliases;
	var $regexp;
	var $required;
	var $default;
	var $value;
  // Has different meaning:
	// module stays enabled	if option is missing
	var $enabled;
	var $visible;

	// By default no regular expression, not required and no default value
	function Option($names, $regexp="", $required=false, $enabled=true, $default="",$visible=false) {
		$name = split(ALIAS_SEP,$names);
		$this->name = $name[0];
		$this->id = OPTION_ID; // Defined in ModuleBase
		$this->aliases = $names;
		$this->regexp = $regexp;
		$this->required = $required;
		$this->default = $default;
		$this->value = "";
		$this->enabled = $enabled;
		$this->visible = $visible;
		$this->ModuleBase();
	}

	function set_value($value) {
		global $debug;
		//$debug->add_message("Value (".$this->value .") changed to $value",$this->name);
		$debug->add(DB_FLOW,$this->name,"Value (%s) changed to %s",$this->value,$value);
		$this->value=$value;
	}

	function set_visible($visible=true) {
		$this->visible=$visible;
	}

	function check_existence($vars) {
		global $error,$debug,$formHandler;
		$keys_array = array_keys($vars);
		$found = false;
		if (!strpos($this->aliases,ALIAS_SEP)===false) {
			$tmp_aliases = split(ALIAS_SEP,$this->aliases);
			while (list(,$alias) = each($tmp_aliases)) {
				if (empty($this->value)) {
					if (in_array($alias,$keys_array)) {
						$found=true;
						if ($this->name != $alias) {
							//$debug->add_message("Change optionname ".$this->name." to ".$alias,$this->name,3);
							$debug->add(DB_XFLOW,$this->name,"Change optionname %s to %s",$this->name,$alias);
							$this->name = $alias;
						}
						$this->value = $vars[$this->name];
						break;
					} //end if in_array
				} // end if empty
			} //end while
		} //end if strpos
		elseif (in_array($this->name,$keys_array)) {
			$found=true;
			$this->value = $vars[$this->name];
		}
		elseif (!empty($this->value)) {
			$found=true;
		}

		if ($found) {
			if (!$this->visible) $formHandler->del_var($this->name);
			return $found;
		} elseif ($this->required) {
			if ($this->name=="recipient") {
				$error->add_message("<p>You probably used a wrong form or entrance to this script.<br> Please read the documentation of the formhandler script.<br> You can find it in the package on http://formhandler.sourceforge.net<p>","See documentation",2);
			}
			$error->add_message("Required option ".$this->name." missing",$this->name,3);
		} elseif ($this->default!="") {
			$this->value = $this->default;
			//$debug->add_message("Option set to default".$this->name,$this->name,3);
			$debug->add(DB_FLOW,$this->name,"Option set to default %s",$this->name);
			return true;
		} //end if found

		return $found;
	}

	function check_value() {
		global $error;
		if ($this->regexp) {
			if (ereg($this->regexp,$this->value)) {
				return true;
			} else {
				$error->add_message("Option ".$this->name ." is not valid",$this->name,3);
				return false;
			} // end if ereg
		} // end if this
	} // end function

} // end class Option

